@@ -34,9 +34,12 @@
 #include <unistd.h>
 #include <signal.h>
 #include <fcntl.h>
-#include <sys/dir.h>
+#include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef WITH_SYSTEMD
+#include <systemd/sd-login.h>
+#endif
 
 // sometimes it's easier to do this manually, w/o gcc helping
 #ifdef PROF
@@ -87,11 +90,20 @@ static inline void free_acquired (proc_t *p, int reuse) {
 #ifdef QUICK_THREADS
     if (!IS_THREAD(p)) {
 #endif
-        if (p->environ) free((void*)*p->environ);
-        if (p->cmdline) free((void*)*p->cmdline);
-        if (p->cgroup)  free((void*)*p->cgroup);
-        if (p->supgid)  free(p->supgid);
-        if (p->supgrp)  free(p->supgrp);
+        if (p->environ)  free((void*)*p->environ);
+        if (p->cmdline)  free((void*)*p->cmdline);
+        if (p->cgroup)   free((void*)*p->cgroup);
+        if (p->supgid)   free(p->supgid);
+        if (p->supgrp)   free(p->supgrp);
+#ifdef WITH_SYSTEMD
+        if (p->sd_mach)  free(p->sd_mach);
+        if (p->sd_ouid)  free(p->sd_ouid);
+        if (p->sd_seat)  free(p->sd_seat);
+        if (p->sd_sess)  free(p->sd_sess);
+        if (p->sd_slice) free(p->sd_slice);
+        if (p->sd_unit)  free(p->sd_unit);
+        if (p->sd_uunit) free(p->sd_uunit);
+#endif
 #ifdef QUICK_THREADS
     }
 #endif
@@ -457,6 +469,81 @@ static void oomadj2proc(const char* S, proc_t *restrict P)
 #endif
 ///////////////////////////////////////////////////////////////////////
 
+static const char *ns_names[] = {
+    [IPCNS] = "ipc",
+    [MNTNS] = "mnt",
+    [NETNS] = "net",
+    [PIDNS] = "pid",
+    [USERNS] = "user",
+    [UTSNS] = "uts",
+};
+
+const char *get_ns_name(int id) {
+    if (id >= NUM_NS)
+        return NULL;
+    return ns_names[id];
+}
+
+int get_ns_id(const char *name) {
+    int i;
+
+    for (i = 0; i < NUM_NS; i++)
+        if (!strcmp(ns_names[i], name))
+            return i;
+    return -1;
+}
+
+static void ns2proc(const char *directory, proc_t *restrict p) {
+    char path[PROCPATHLEN];
+    struct stat sb;
+    int i;
+
+    for (i = 0; i < NUM_NS; i++) {
+        snprintf(path, sizeof(path), "%s/ns/%s", directory, ns_names[i]);
+        if (0 == stat(path, &sb))
+            p->ns[i] = (long)sb.st_ino;
+#if 0
+        else                           // this allows a caller to distinguish
+            p->ns[i] = -errno;         // between the ENOENT or EACCES errors
+#endif
+    }
+}
+
+#ifdef WITH_SYSTEMD
+static void sd2proc(proc_t *restrict p) {
+    char buf[64];
+    uid_t uid;
+
+    if (0 > sd_pid_get_machine_name(p->tid, &p->sd_mach))
+        p->sd_mach = strdup("-");
+
+    if (0 > sd_pid_get_owner_uid(p->tid, &uid))
+        p->sd_ouid = strdup("-");
+    else {
+        snprintf(buf, sizeof(buf), "%d", (int)uid);
+        p->sd_ouid = strdup(buf);
+    }
+
+    if (0 > sd_pid_get_session(p->tid, &p->sd_sess)) {
+        p->sd_sess = strdup("-");
+        p->sd_seat = strdup("-");
+    } else {
+        if (0 > sd_session_get_seat(p->sd_sess, &p->sd_seat))
+            p->sd_seat = strdup("-");
+    }
+
+    if (0 > sd_pid_get_slice(p->tid, &p->sd_slice))
+        p->sd_slice = strdup("-");
+
+    if (0 > sd_pid_get_unit(p->tid, &p->sd_unit))
+        p->sd_unit = strdup("-");
+
+    if (0 > sd_pid_get_user_unit(p->tid, &p->sd_uunit))
+        p->sd_uunit = strdup("-");
+}
+#endif
+///////////////////////////////////////////////////////////////////////
+
 
 // Reads /proc/*/stat files, being careful not to trip over processes with
 // names like ":-) 1 2 3 4 5 6".
@@ -533,29 +620,28 @@ static void statm2proc(const char* s, proc_t *restrict P) {
 }
 
 static int file2str(const char *directory, const char *what, struct utlbuf_s *ub) {
- #define readMAX  4096
- #define buffMIN (tot_read + num + 1)  // +1 for the '\0' delimiter
-    char path[PROCPATHLEN], chunk[readMAX];
+ #define buffGRW 1024
+    char path[PROCPATHLEN];
     int fd, num, tot_read = 0;
 
     /* on first use we preallocate a buffer of minimum size to emulate
        former 'local static' behavior -- even if this read fails, that
-       buffer will likely soon be used for another sudirectory anyway */
+       buffer will likely soon be used for another subdirectory anyway
+       ( besides, with this xcalloc we will never need to use memcpy ) */
     if (ub->buf) ub->buf[0] = '\0';
-    else ub->buf = xcalloc((ub->siz = readMAX));
+    else ub->buf = xcalloc((ub->siz = buffGRW));
     sprintf(path, "%s/%s", directory, what);
     if (-1 == (fd = open(path, O_RDONLY, 0))) return -1;
-    while (0 < (num = read(fd, chunk, readMAX))) {
-        if (ub->siz < buffMIN)
-            ub->buf = xrealloc(ub->buf, (ub->siz = buffMIN));
-        memcpy(ub->buf + tot_read, chunk, num);
+    while (0 < (num = read(fd, ub->buf + tot_read, ub->siz - tot_read))) {
         tot_read += num;
+        if (tot_read < ub->siz) break;
+        ub->buf = xrealloc(ub->buf, (ub->siz += buffGRW));
     };
     ub->buf[tot_read] = '\0';
     close(fd);
+    if (unlikely(tot_read < 1)) return -1;
     return tot_read;
- #undef readMAX
- #undef buffMIN
+ #undef buffGRW
 }
 
 static char** file2strvec(const char* directory, const char* what) {
@@ -646,6 +732,7 @@ static int read_unvectored(char *restrict const dst, unsigned sz, const char* wh
     close(fd);
     if(n){
         int i=n;
+        while(i && dst[i-1]=='\0') --i; // skip trailing zeroes
         while(i--)
             if(dst[i]=='\n' || dst[i]=='\0') dst[i]=sep;
         if(dst[n-1]==' ') dst[n-1]='\0';
@@ -729,6 +816,64 @@ int read_cmdline(char *restrict const dst, unsigned sz, unsigned pid) {
 }
 
 
+    // Provide the means to value proc_t.lxcname (perhaps only with "-") while
+    // tracking all names already seen thus avoiding the overhead of repeating
+    // malloc() and free() calls.
+static const char *lxc_containers (const char *path) {
+    static struct utlbuf_s ub = { NULL, 0 };   // util buffer for whole cgroup
+    static char lxc_none[] = "-";
+    /*
+       try to locate the lxc delimiter eyecatcher somewhere in a task's cgroup
+       directory -- the following are from nested privileged plus unprivileged
+       containers, where the '/lxc/' delimiter precedes the container name ...
+           10:cpuset:/lxc/lxc-P/lxc/lxc-P-nested
+           10:cpuset:/user.slice/user-1000.slice/session-c2.scope/lxc/lxc-U/lxc/lxc-U-nested
+
+       ... some minor complications are the potential addition of more cgroups
+       for a controller displacing the lxc name (normally last on a line), and
+       environments with unexpected /proc/##/cgroup ordering/contents as with:
+           10:cpuset:/lxc/lxc-P/lxc/lxc-P-nested/MY-NEW-CGROUP
+       or
+           2:name=systemd:/
+           1:cpuset,cpu,cpuacct,devices,freezer,net_cls,blkio,perf_event,net_prio:/lxc/lxc-P
+    */
+    if (file2str(path, "cgroup", &ub) > 0) {
+        static const char lxc_delm[] = "/lxc/";
+        char *p1;
+
+        if ((p1 = strstr(ub.buf, lxc_delm))) {
+            static struct lxc_ele {
+                struct lxc_ele *next;
+                const char *name;
+            } *anchor = NULL;
+            struct lxc_ele *ele = anchor;
+            char *p2;
+
+            if ((p2 = strchr(p1, '\n')))       // isolate a controller's line
+                *p2 = '\0';
+            do {                               // deal with nested containers
+                p2 = p1 + (sizeof(lxc_delm)-1);
+                p1 = strstr(p2, lxc_delm);
+            } while (p1);
+            if ((p1 = strchr(p2, '/')))        // isolate name only substring
+                *p1 = '\0';
+            while (ele) {                      // have we already seen a name
+                if (!strcmp(ele->name, p2))
+                    return ele->name;          // return just a recycled name
+                ele = ele->next;
+            }
+            ele = (struct lxc_ele *)xmalloc(sizeof(struct lxc_ele));
+            ele->name = xstrdup(p2);
+            ele->next = anchor;                // push the new container name
+            anchor = ele;
+            return ele->name;                  // return a new container name
+        }
+    }
+    return lxc_none;
+}
+///////////////////////////////////////////////////////////////////////
+
+
 /* These are some nice GNU C expression subscope "inline" functions.
  * The can be used with arbitrary types and evaluate their arguments
  * exactly once.
@@ -845,6 +990,17 @@ static proc_t* simple_readproc(PROCTAB *restrict const PT, proc_t *restrict cons
     }
 #endif
 
+    if (unlikely(flags & PROC_FILLNS))          // read /proc/#/ns/*
+        ns2proc(path, p);
+
+#ifdef WITH_SYSTEMD
+    if (unlikely(flags & PROC_FILLSYSTEMD))     // get sd-login.h stuff
+        sd2proc(p);
+#endif
+
+    if (unlikely(flags & PROC_FILL_LXC))        // value the lxc name
+        p->lxcname = lxc_containers(path);
+
     return p;
 next_proc:
     return NULL;
@@ -948,6 +1104,14 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
         } else
             t->cgroup = NULL;
 
+#ifdef WITH_SYSTEMD
+        if (unlikely(flags & PROC_FILLSYSTEMD))         // get sd-login.h stuff
+            sd2proc(t);
+#endif
+
+        if (unlikely(flags & PROC_FILL_LXC))            // value the lxc name
+            t->lxcname = lxc_containers(path);
+
 #ifdef QUICK_THREADS
     } else {
         t->size     = p->size;
@@ -963,6 +1127,16 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
         if (t->supgid) free(t->supgid);
         t->supgid   = p->supgid;
         t->supgrp   = p->supgrp;
+#ifdef WITH_SYSTEMD
+        t->sd_mach  = p->sd_mach;
+        t->sd_ouid  = p->sd_ouid;
+        t->sd_seat  = p->sd_seat;
+        t->sd_sess  = p->sd_sess;
+        t->sd_slice = p->sd_slice;
+        t->sd_unit  = p->sd_unit;
+        t->sd_uunit = p->sd_uunit;
+#endif
+        t->lxcname = p->lxcname;
         MK_THREAD(t);
     }
 #endif
@@ -976,6 +1150,9 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
     }
 #endif
 
+    if (unlikely(flags & PROC_FILLNS))                  // read /proc/#/task/#/ns/*
+        ns2proc(path, t);
+
     return t;
 next_task:
     return NULL;
@@ -988,7 +1165,7 @@ next_task:
 // This finds processes in /proc in the traditional way.
 // Return non-zero on success.
 static int simple_nextpid(PROCTAB *restrict const PT, proc_t *restrict const p) {
-  static struct direct *ent;		/* dirent handle */
+  static struct dirent *ent;		/* dirent handle */
   char *restrict const path = PT->path;
   for (;;) {
     ent = readdir(PT->procfs);
@@ -1006,7 +1183,7 @@ static int simple_nextpid(PROCTAB *restrict const PT, proc_t *restrict const p)
 // This finds tasks in /proc/*/task/ in the traditional way.
 // Return non-zero on success.
 static int simple_nexttid(PROCTAB *restrict const PT, const proc_t *restrict const p, proc_t *restrict const t, char *restrict const path) {
-  static struct direct *ent;		/* dirent handle */
+  static struct dirent *ent;		/* dirent handle */
   if(PT->taskdir_user != p->tgid){
     if(PT->taskdir){
       closedir(PT->taskdir);
