@@ -36,6 +36,7 @@
 
 #include "c.h"
 #include "fileutils.h"
+#include "nsutils.h"
 #include "strutils.h"
 #include "nls.h"
 #include "xalloc.h"
@@ -43,6 +44,7 @@
 #include "proc/sig.h"
 #include "proc/devname.h"
 #include "proc/procps.h"	/* char *user_from_uid(uid_t uid) */
+#include "proc/readproc.h"
 #include "proc/version.h"	/* procps_version */
 #include "rpmatch.h"
 
@@ -56,11 +58,14 @@ struct run_time_conf_t {
 	int noaction;
 	int debugging;
 };
-static int tty_count, uid_count, cmd_count, pid_count;
+static int tty_count, uid_count, cmd_count, pid_count, namespace_count;
 static int *ttys;
 static uid_t *uids;
 static const char **cmds;
 static int *pids;
+static char **namespaces;
+static int ns_pid;
+static proc_t ns_task;
 
 #define ENLIST(thing,addme) do{ \
 if(!thing##s) thing##s = xmalloc(sizeof(*thing##s)*saved_argc); \
@@ -85,6 +90,39 @@ static void display_kill_version(void)
 	fprintf(stdout, PROCPS_NG_VERSION);
 }
 
+static int ns_flags = 0x3f;
+static int parse_namespaces(char *optarg)
+{
+	char *ptr = optarg, *tmp;
+	int len, id;
+
+	ns_flags = 0;
+	while (1) {
+		if (strchr(ptr, ',') == NULL) {
+			len = -1;
+			tmp = strdup(ptr);
+		} else {
+			len = strchr(ptr, ',') - ptr;
+			tmp = strndup(ptr, len);
+		}
+
+		id = get_ns_id(tmp);
+		if (id == -1) {
+			fprintf(stderr, "%s is not a valid namespace\n", tmp);
+			free(tmp);
+			return 1;
+		}
+		ns_flags |= (1 << id);
+		ENLIST(namespace, tmp);
+
+		if (len == -1)
+			break;
+
+		ptr+= len + 1;
+	}
+	return 0;
+}
+
 /* kill or nice a process */
 static void hurt_proc(int tty, int uid, int pid, const char *restrict const cmd,
 		      struct run_time_conf_t *run_time)
@@ -131,6 +169,7 @@ static void check_proc(int pid, struct run_time_conf_t *run_time)
 {
 	char buf[128];
 	struct stat statbuf;
+	proc_t task;
 	char *tmp;
 	int tty;
 	int fd;
@@ -156,7 +195,8 @@ static void check_proc(int pid, struct run_time_conf_t *run_time)
 		if (i == -1)
 			goto closure;
 	}
-	read(fd, buf, 128);
+	if (read(fd, buf, 128) <= 0)
+	    goto closure;
 	buf[127] = '\0';
 	tmp = strrchr(buf, ')');
 	*tmp++ = '\0';
@@ -183,6 +223,16 @@ static void check_proc(int pid, struct run_time_conf_t *run_time)
 		if (i == -1)
 			goto closure;
 	}
+	if (ns_pid) {
+		if (ns_read(pid, &task))
+			goto closure;
+		for (i = 0; i < NUM_NS; i++) {
+			if (ns_flags & (1 << i)) {
+				if (task.ns[i] != ns_task.ns[i])
+					goto closure;
+			}
+		}
+	}
 	/* This is where we kill/nice something. */
 	/* for debugging purposes?
 	fprintf(stderr, "PID %d, UID %d, TTY %d,%d, COMM %s\n",
@@ -275,11 +325,11 @@ static void __attribute__ ((__noreturn__)) kill_usage(FILE * out)
 	fprintf(out,
               _(" %s [options] <pid> [...]\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" <pid> [...]            send signal to every <pid> listed\n"
-		" -<signal>, -s, --signal <signal>\n"
-		"                        specify the <signal> to be sent\n"
-		" -l, --list=[<signal>]  list all signal names, or convert one to a name\n"
-		" -L, --table            list all signal names in a nice table\n"), out);
+	fputs(_(" <pid> [...]            send signal to every <pid> listed\n"), out);
+	fputs(_(" -<signal>, -s, --signal <signal>\n"
+		"                        specify the <signal> to be sent\n"), out);
+	fputs(_(" -l, --list=[<signal>]  list all signal names, or convert one to a name\n"), out);
+	fputs(_(" -L, --table            list all signal names in a nice table\n"), out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_HELP, out);
 	fputs(USAGE_VERSION, out);
@@ -302,20 +352,29 @@ static void __attribute__ ((__noreturn__)) skillsnice_usage(FILE * out)
 			program_invocation_short_name);
 	}
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -f, --fast         fast mode (not implemented)\n"
-		" -i, --interactive  interactive\n"
-		" -l, --list         list all signal names\n"
-		" -L, --table        list all signal names in a nice table\n"
-		" -n, --no-action    no action\n"
-		" -v, --verbose      explain what is being done\n"
-		" -w, --warnings     enable warnings (not implemented)\n"), out);
+	fputs(_(" -f, --fast         fast mode (not implemented)\n"), out);
+	fputs(_(" -i, --interactive  interactive\n"), out);
+	fputs(_(" -l, --list         list all signal names\n"), out);
+	fputs(_(" -L, --table        list all signal names in a nice table\n"), out);
+	fputs(_(" -n, --no-action    do not actually kill processes; just print what would happen\n"), out);
+	fputs(_(" -v, --verbose      explain what is being done\n"), out);
+	fputs(_(" -w, --warnings     enable warnings (not implemented)\n"), out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(_("Expression can be: terminal, user, pid, command.\n"
-		"The options below may be used to ensure correct interpretation.\n"
-		" -c, --command <command>  expression is a command name\n"
-		" -p, --pid <pid>          expression is a process id number\n"
-		" -t, --tty <tty>          expression is a terminal\n"
-		" -u, --user <username>    expression is a username\n"), out);
+		"The options below may be used to ensure correct interpretation.\n"), out);
+	fputs(_(" -c, --command <command>  expression is a command name\n"), out);
+	fputs(_(" -p, --pid <pid>          expression is a process id number\n"), out);
+	fputs(_(" -t, --tty <tty>          expression is a terminal\n"), out);
+	fputs(_(" -u, --user <username>    expression is a username\n"), out);
+	fputs(USAGE_SEPARATOR, out);
+	fputs(_("Alternatively, expression can be:\n"), out);
+	fputs(_(" --ns <pid>               match the processes that belong to the same\n"
+		"                          namespace as <pid>\n"), out);
+	fputs(_(" --nslist <ns,...>        list which namespaces will be considered for\n"
+		"                          the --ns option; available namespaces are\n:"
+	        "                          ipc, mnt, net, pid, user, uts\n"), out);
+
+	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_HELP, out);
 	fputs(USAGE_VERSION, out);
@@ -339,17 +398,15 @@ static void __attribute__ ((__noreturn__)) skillsnice_usage(FILE * out)
 
 static int skill_sig_option(int *argc, char **argv)
 {
-	int i, nargs = *argc;
+	int i;
 	int signo = -1;
-	for (i = 1; i < nargs; i++) {
+	for (i = 1; i < *argc; i++) {
 		if (argv[i][0] == '-') {
 			signo = signal_name_to_number(argv[i] + 1);
 			if (-1 < signo) {
-				if (nargs - i) {
-					nargs--;
-					memmove(argv + i, argv + i + 1,
-						sizeof(char *) * (nargs - i));
-				}
+				memmove(argv + i, argv + i + 1,
+					sizeof(char *) * (*argc - i));
+				(*argc)--;
 				return signo;
 			}
 		}
@@ -362,7 +419,6 @@ static void __attribute__ ((__noreturn__))
     kill_main(int argc, char **argv)
 {
 	int signo, i;
-	int sigopt = 0;
 	int loop = 1;
 	long pid;
 	int exitvalue = EXIT_SUCCESS;
@@ -387,8 +443,6 @@ static void __attribute__ ((__noreturn__))
 	signo = skill_sig_option(&argc, argv);
 	if (signo < 0)
 		signo = SIGTERM;
-	else
-		sigopt++;
 
 	opterr=0; /* suppress errors on -123 */
 	while (loop == 1 && (i = getopt_long(argc, argv, "l::Ls:hV", longopts, NULL)) != -1)
@@ -422,6 +476,13 @@ static void __attribute__ ((__noreturn__))
 			if (!isdigit(optopt)) {
 				xwarnx(_("invalid argument %c"), optopt);
 				kill_usage(stderr);
+			} else {
+			    /* Special case for signal digit negative
+			     * PIDs */
+			    pid = (long)('0' - optopt);
+			    if (kill((pid_t)pid, signo) != 0)
+				exitvalue = EXIT_FAILURE;
+			    exit(exitvalue);
 			}
 			loop=0;
 			break;
@@ -429,9 +490,12 @@ static void __attribute__ ((__noreturn__))
 			kill_usage(stderr);
 		}
 
-	argc -= optind + sigopt;
+	argc -= optind;
 	argv += optind;
 
+	if (argc < 1)
+		kill_usage(stderr);
+
 	for (i = 0; i < argc; i++) {
 		pid = strtol_or_err(argv[i], _("failed to parse argument"));
 		if (!kill((pid_t) pid, signo))
@@ -488,6 +552,11 @@ static void skillsnice_parse(int argc,
 	int prino = DEFAULT_NICE;
 	int ch, i;
 
+	enum {
+		NS_OPTION = CHAR_MAX + 1,
+		NSLIST_OPTION,
+	};
+
 	static const struct option longopts[] = {
 		{"command", required_argument, NULL, 'c'},
 		{"debug", no_argument, NULL, 'd'},
@@ -499,6 +568,8 @@ static void skillsnice_parse(int argc,
 		{"table", no_argument, NULL, 'L'},
 		{"tty", required_argument, NULL, 't'},
 		{"user", required_argument, NULL, 'u'},
+		{"ns", required_argument, NULL, NS_OPTION},
+		{"nslist", required_argument, NULL, NSLIST_OPTION},
 		{"verbose", no_argument, NULL, 'v'},
 		{"warnings", no_argument, NULL, 'w'},
 		{"help", no_argument, NULL, 'h'},
@@ -572,6 +643,25 @@ static void skillsnice_parse(int argc,
 				}
 			}
 			break;
+		case NS_OPTION:
+			ns_pid = atoi(optarg);
+			if (ns_pid == 0) {
+				xwarnx(_("invalid pid number %s"), optarg);
+				kill_usage(stderr);
+			}
+			if (ns_read(ns_pid, &ns_task)) {
+				xwarnx(_("error reading reference namespace "
+					 "information"));
+				kill_usage(stderr);
+			}
+
+			break;
+		case NSLIST_OPTION:
+			if (parse_namespaces(optarg)) {
+				xwarnx(_("invalid namespace list"));
+				kill_usage(stderr);
+			}
+			break;
 		case 'v':
 			run_time->verbose = 1;
 			break;
@@ -605,7 +695,7 @@ static void skillsnice_parse(int argc,
 	}
 
 	/* No more arguments to process. Must sanity check. */
-	if (!tty_count && !uid_count && !cmd_count && !pid_count)
+	if (!tty_count && !uid_count && !cmd_count && !pid_count && !ns_pid)
 		xerrx(EXIT_FAILURE, _("no process selection criteria"));
 	if ((run_time->fast | run_time->interactive | run_time->
 	     verbose | run_time->warnings | run_time->noaction) & ~1)
@@ -659,7 +749,7 @@ int main(int argc, char ** argv)
 		kill_main(argc, argv);
 		break;
 	default:
-		fprintf(stderr, _("skill: \"%s\" is not support\n"),
+		fprintf(stderr, _("skill: \"%s\" is not supported\n"),
 			program_invocation_short_name);
 		fprintf(stderr, USAGE_MAN_TAIL("skill(1)"));
 		return EXIT_FAILURE;
