@@ -70,6 +70,7 @@ typedef struct utmp utmp_t;
 #endif
 
 #define MAX_CMD_WIDTH	512
+#define MIN_CMD_WIDTH   7
 
 /*
  * This routine is careful since some programs leave utmp strings
@@ -100,6 +101,10 @@ static void print_host(const char *restrict host, int len, const int fromlen)
 	 * space-fill, and a '-' too if needed to ensure the
 	 * column exists
 	 */
+	if (!width) {
+		fputc('-', stdout);
+		++width;
+	}
 	while (width++ < fromlen)
 		fputc(' ', stdout);
 }
@@ -236,10 +241,12 @@ static void print_time_ival7(time_t t, int centi_sec, FILE * fout)
 			fprintf(fout, _(" %2ludays"), t / (24 * 60 * 60));
 		else if (t >= 60 * 60)
 			/* > 1 hour */
+		        /* Translation Hint: Hours:Minutes */
 			fprintf(fout, " %2lu:%02u ", t / (60 * 60),
 				(unsigned)((t / 60) % 60));
 		else if (t > 60)
 			/* > 1 minute */
+		        /* Translation Hint: Minutes:Seconds */
 			fprintf(fout, _(" %2lu:%02um"), t / 60, (unsigned)t % 60);
 		else
 			fprintf(fout, "       ");
@@ -249,12 +256,15 @@ static void print_time_ival7(time_t t, int centi_sec, FILE * fout)
 			fprintf(fout, _(" %2ludays"), t / (24 * 60 * 60));
 		else if (t >= 60 * 60)
 			/* 1 hour or more */
+		        /* Translation Hint: Hours:Minutes */
 			fprintf(fout, _(" %2lu:%02um"), t / (60 * 60),
 				(unsigned)((t / 60) % 60));
 		else if (t > 60)
 			/* 1 minute or more */
+		        /* Translation Hint: Minutes:Seconds */
 			fprintf(fout, " %2lu:%02u ", t / 60, (unsigned)t % 60);
 		else
+		        /* Translation Hint: Seconds:Centiseconds */
 			fprintf(fout, _(" %2lu.%02us"), t, centi_sec);
 	}
 }
@@ -338,7 +348,8 @@ static const proc_t *getproc(const utmp_t * restrict const u,
 		const proc_t *restrict const tmp = *pptr;
 		if (unlikely(tmp->tgid == u->ut_pid)) {
 			*found_utpid = 1;
-			best = tmp;
+            if (!best)
+                best = tmp;
 		}
 		if (tmp->tty != line)
 			continue;
@@ -437,12 +448,12 @@ static void __attribute__ ((__noreturn__))
 	fprintf(out,
               _(" %s [options]\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -h, --no-header     do not print header\n"
-		" -u, --no-current    ignore current process username\n"
-		" -s, --short         short format\n"
-		" -f, --from          show remote hostname field\n"
-		" -o, --old-style     old style output\n"
-		" -i, --ip-addr       display IP address instead of hostname (if possible)\n"), out);
+	fputs(_(" -h, --no-header     do not print header\n"),out);
+	fputs(_(" -u, --no-current    ignore current process username\n"),out);
+	fputs(_(" -s, --short         short format\n"),out);
+	fputs(_(" -f, --from          show remote hostname field\n"),out);
+	fputs(_(" -o, --old-style     old style output\n"),out);
+	fputs(_(" -i, --ip-addr       display IP address instead of hostname (if possible)\n"), out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(_("     --help     display this help and exit\n"), out);
 	fputs(USAGE_VERSION, out);
@@ -475,9 +486,10 @@ int main(int argc, char **argv)
 	static const struct option longopts[] = {
 		{"no-header", no_argument, NULL, 'h'},
 		{"no-current", no_argument, NULL, 'u'},
-		{"sort", no_argument, NULL, 's'},
+		{"short", no_argument, NULL, 's'},
 		{"from", no_argument, NULL, 'f'},
 		{"old-style", no_argument, NULL, 'o'},
+		{"ip-addr", no_argument, NULL, 'i'},
 		{"help", no_argument, NULL, HELP_OPTION},
 		{"version", no_argument, NULL, 'V'},
 		{NULL, 0, NULL, 0}
@@ -559,22 +571,17 @@ int main(int argc, char **argv)
 		maxcmd = atoi(p);
 	else
 		maxcmd = MAX_CMD_WIDTH;
-	if (maxcmd < 71)
-		xerrx(EXIT_FAILURE, _("%d column window is too narrow"), maxcmd);
-	if (MAX_CMD_WIDTH < maxcmd) {
-		xwarnx(_("%d column width exceeds command buffer size, truncating to %d"),
-		       maxcmd, MAX_CMD_WIDTH);
+	if (MAX_CMD_WIDTH < maxcmd)
 		maxcmd = MAX_CMD_WIDTH;
-	}
 	maxcmd -= 21 + userlen + (from ? fromlen : 0) + (longform ? 20 : 0);
-	if (maxcmd < 3)
-		xwarnx(_("warning: screen width %d suboptimal"), win.ws_col);
+	if (maxcmd < MIN_CMD_WIDTH)
+        maxcmd = MIN_CMD_WIDTH;
 
 	procs = readproctab(PROC_FILLCOM | PROC_FILLUSR | PROC_FILLSTAT);
 
 	if (header) {
 		/* print uptime and headers */
-		print_uptime();
+		print_uptime(0);
 		/* Translation Hint: Following five uppercase messages are
 		 * headers. Try to keep alignment intact.  */
 		printf(_("%-*s TTY      "), userlen, _("USER"));
