@@ -31,15 +31,13 @@
 #include "config.h"
 #include "fileutils.h"
 #include "nls.h"
-#include "proc/procps.h"
 #include "strutils.h"
 #include "xalloc.h"
 #include <ctype.h>
 #include <errno.h>
-#include <errno.h>
 #include <getopt.h>
 #include <locale.h>
-#include <locale.h>
+#include <limits.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,10 +46,10 @@
 #include <sys/time.h>
 #include <sys/wait.h>
 #include <termios.h>
-#include <termios.h>
 #include <time.h>
 #include <unistd.h>
 #ifdef WITH_WATCH8BIT
+# define _XOPEN_SOURCE_EXTENDED 1
 # include <wchar.h>
 # include <wctype.h>
 # include <ncursesw/ncurses.h>
@@ -82,7 +80,7 @@ static int show_title = 2;	/* number of lines used, 2 or 0 */
 static int precise_timekeeping = 0;
 
 #define min(x,y) ((x) > (y) ? (y) : (x))
-#define MAX_ANSIBUF 10
+#define MAX_ANSIBUF 100
 
 static void __attribute__ ((__noreturn__))
     usage(FILE * out)
@@ -91,16 +89,16 @@ static void __attribute__ ((__noreturn__))
 	fprintf(out,
               _(" %s [options] command\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
-	fputs(_("  -b, --beep             beep if command has a non-zero exit\n"
-		"  -c, --color            interpret ANSI color sequences\n"
-		"  -d, --differences[=<permanent>]\n"
-                "                         highlight changes between updates\n"
-		"  -e, --errexit          exit if command has a non-zero exit\n"
-		"  -g, --chgexit          exit when output from command changes\n"
-		"  -n, --interval <secs>  seconds to wait between updates\n"
-		"  -p, --precise          attempt run command in precise intervals\n"
-		"  -t, --no-title         turn off header\n"
-		"  -x, --exec             pass command to exec instead of \"sh -c\"\n"), out);
+	fputs(_("  -b, --beep             beep if command has a non-zero exit\n"), out);
+	fputs(_("  -c, --color            interpret ANSI color and style sequences\n"), out);
+	fputs(_("  -d, --differences[=<permanent>]\n"
+                "                         highlight changes between updates\n"), out);
+	fputs(_("  -e, --errexit          exit if command has a non-zero exit\n"), out);
+	fputs(_("  -g, --chgexit          exit when output from command changes\n"), out);
+	fputs(_("  -n, --interval <secs>  seconds to wait between updates\n"), out);
+	fputs(_("  -p, --precise          attempt run command in precise intervals\n"), out);
+	fputs(_("  -t, --no-title         turn off header\n"), out);
+	fputs(_("  -x, --exec             pass command to exec instead of \"sh -c\"\n"), out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_HELP, out);
 	fputs(_(" -v, --version  output version information and exit\n"), out);
@@ -109,41 +107,99 @@ static void __attribute__ ((__noreturn__))
 	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
+static int nr_of_colors;
+static int attributes;
+static int fg_col;
+static int bg_col;
+
+
+static void reset_ansi(void)
+{
+	attributes = A_NORMAL;
+	fg_col = 0;
+	bg_col = 0;
+}
+
 static void init_ansi_colors(void)
 {
-	int i;
 	short ncurses_colors[] = {
-		COLOR_BLACK, COLOR_RED, COLOR_GREEN, COLOR_YELLOW, COLOR_BLUE,
-		COLOR_MAGENTA, COLOR_CYAN, COLOR_WHITE
+		-1, COLOR_BLACK, COLOR_RED, COLOR_GREEN, COLOR_YELLOW,
+		COLOR_BLUE, COLOR_MAGENTA, COLOR_CYAN, COLOR_WHITE
 	};
 
-	for (i = 0; i < 8; i++)
-		init_pair(i + 1, ncurses_colors[i], -1);
+	nr_of_colors = sizeof(ncurses_colors) / sizeof(short);
+
+	for (bg_col = 0; bg_col < nr_of_colors; bg_col++)
+		for (fg_col = 0; fg_col < nr_of_colors; fg_col++)
+			init_pair(bg_col * nr_of_colors + fg_col + 1, ncurses_colors[fg_col], ncurses_colors[bg_col]);
+	reset_ansi();
 }
 
+
 static void set_ansi_attribute(const int attrib)
 {
 	switch (attrib) {
-	case -1:
-		return;
-	case 0:
-		standend();
-		return;
-	case 1:
-		attrset(A_BOLD);
-		return;
-	}
-	if (attrib >= 30 && attrib <= 37) {
-		color_set(attrib - 29, NULL);
-		return;
+	case -1:	/* restore last settings */
+		break;
+	case 0:		/* restore default settings */
+		reset_ansi();
+		break;
+	case 1:		/* set bold / increased intensity */
+		attributes |= A_BOLD;
+		break;
+	case 2:		/* set decreased intensity (if supported) */
+		attributes |= A_DIM;
+		break;
+#ifdef A_ITALIC
+	case 3:		/* set italic (if supported) */
+		attributes |= A_ITALIC;
+		break;
+#endif
+	case 4:		/* set underline */
+		attributes |= A_UNDERLINE;
+		break;
+	case 5:		/* set blinking */
+		attributes |= A_BLINK;
+		break;
+	case 7:		/* set inversed */
+		attributes |= A_REVERSE;
+		break;
+	case 21:	/* unset bold / increased intensity */
+		attributes &= ~A_BOLD;
+		break;
+	case 22:	/* unset bold / any intensity modifier */
+		attributes &= ~(A_BOLD | A_DIM);
+		break;
+#ifdef A_ITALIC
+	case 23:	/* unset italic */
+		attributes &= ~A_ITALIC;
+		break;
+#endif
+	case 24:	/* unset underline */
+		attributes &= ~A_UNDERLINE;
+		break;
+	case 25:	/* unset blinking */
+		attributes &= ~A_BLINK;
+		break;
+	case 27:	/* unset inversed */
+		attributes &= ~A_REVERSE;
+		break;
+	default:
+		if (attrib >= 30 && attrib <= 37) {	/* set foreground color */
+			fg_col = attrib - 30 + 1;
+		} else if (attrib >= 40 && attrib <= 47) { /* set background color */
+			bg_col = attrib - 40 + 1;
+		}
 	}
+
+	attrset(attributes | COLOR_PAIR(bg_col * nr_of_colors + fg_col + 1));
 }
 
 static void process_ansi(FILE * fp)
 {
-	int i, c, num1, num2;
+	int i, c;
 	char buf[MAX_ANSIBUF];
-	char *nextnum;
+	char *numstart, *endptr;
 
 	c = getc(fp);
 	if (c != '[') {
@@ -164,13 +220,18 @@ static void process_ansi(FILE * fp)
 		}
 		buf[i] = (char)c;
 	}
-	num1 = strtol(buf, &nextnum, 10);
-	if (nextnum != buf && nextnum[0] != '\0')
-		num2 = strtol(nextnum + 1, NULL, 10);
-	else
-		num2 = -1;
-	set_ansi_attribute(num1);
-	set_ansi_attribute(num2);
+	/*
+	 * buf now contains a semicolon-separated list of decimal integers,
+	 * each indicating an attribute to apply.
+	 * For example, buf might contain "0;1;31", derived from the color
+	 * escape sequence "<ESC>[0;1;31m". There can be 1 or more
+	 * attributes to apply, but typically there are between 1 and 3.
+	 */
+
+	if (*endptr == '\0') set_ansi_attribute(0); /* [m treated as [0m */
+
+	for (endptr = numstart = buf; *endptr != '\0'; numstart = endptr + 1)
+		set_ansi_attribute(strtol(numstart, &endptr, 10));
 }
 
 static void __attribute__ ((__noreturn__)) do_exit(int status)
@@ -390,6 +451,7 @@ static int run_command(char *restrict command, char **restrict command_argv)
 		if (dup2(pipefd[1], 1) < 0) {	/* replace stdout with write side of pipe */
 			xerr(3, _("dup2 failed"));
 		}
+		close(pipefd[1]);		/* once duped, the write fd isn't needed */
 		dup2(1, 2);			/* stderr should default to stdout */
 
 		if (flags & WATCH_EXEC) {	/* pass command to exec instead of system */
@@ -413,8 +475,11 @@ static int run_command(char *restrict command, char **restrict command_argv)
 	if ((p = fdopen(pipefd[0], "r")) == NULL)
 		xerr(5, _("fdopen"));
 
+	reset_ansi();
 	for (y = show_title; y < height; y++) {
-		int eolseen = 0, tabpending = 0;
+		int eolseen = 0, tabpending = 0, tabwaspending = 0;
+		if (flags & WATCH_COLOR)
+			set_ansi_attribute(-1);
 #ifdef WITH_WATCH8BIT
 		wint_t carry = WEOF;
 #endif
@@ -426,6 +491,10 @@ static int run_command(char *restrict command, char **restrict command_argv)
 #endif
 			int attr = 0;
 
+			if (tabwaspending && (flags & WATCH_COLOR))
+				set_ansi_attribute(-1);
+			tabwaspending = 0;
+
 			if (!eolseen) {
 				/* if there is a tab pending, just
 				 * spit spaces until the next stop
@@ -475,16 +544,25 @@ static int run_command(char *restrict command, char **restrict command_argv)
 					carry = c;	/* character on the next line */
 					continue;	/* because it won't fit here */
 				}
-				if (c == WEOF || c == L'\n' || c == L'\t')
+				if (c == WEOF || c == L'\n' || c == L'\t') {
 					c = L' ';
+					if (flags & WATCH_COLOR)
+						attrset(A_NORMAL);
+				}
 #else
-				if (c == EOF || c == '\n' || c == '\t')
+				if (c == EOF || c == '\n' || c == '\t') {
 					c = ' ';
+					if (flags & WATCH_COLOR)
+						attrset(A_NORMAL);
+				}
 #endif
-				if (tabpending && (((x + 1) % 8) == 0))
+				if (tabpending && (((x + 1) % 8) == 0)) {
 					tabpending = 0;
+					tabwaspending = 1;
+				}
 			}
 			move(y, x);
+
 			if (!first_screen && !exit_early && (flags & WATCH_CHGEXIT)) {
 #ifdef WITH_WATCH8BIT
 				cchar_t oldc;
@@ -538,6 +616,7 @@ static int run_command(char *restrict command, char **restrict command_argv)
 
 	fclose(p);
 
+
 	/* harvest child process and get status, propagated from command */
 	if (waitpid(child, &status, 0) < 0)
 		xerr(8, _("waitpid"));
@@ -590,7 +669,9 @@ int main(int argc, char *argv[])
 		{0, 0, 0, 0}
 	};
 
+#ifdef HAVE_PROGRAM_INVOCATION_NAME
 	program_invocation_name = program_invocation_short_name;
+#endif
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
@@ -627,8 +708,8 @@ int main(int argc, char *argv[])
 			interval = strtod_or_err(optarg, _("failed to parse argument"));
 			if (interval < 0.1)
 				interval = 0.1;
-			if (interval > ~0u / 1000000)
-				interval = ~0u / 1000000;
+			if (interval > UINT_MAX)
+				interval = UINT_MAX;
 			break;
 		case 'p':
 			precise_timekeeping = 1;
@@ -731,13 +812,18 @@ int main(int argc, char *argv[])
 
 		if (run_command(command, command_argv))
 			break;
+
+
 		if (precise_timekeeping) {
 			watch_usec_t cur_time = get_time_usec();
 			next_loop += USECS_PER_SEC * interval;
 			if (cur_time < next_loop)
 				usleep(next_loop - cur_time);
 		} else
-			usleep(interval * 1000000);
+			if (interval < UINT_MAX / USECS_PER_SEC)
+				usleep(interval * USECS_PER_SEC);
+			else
+				sleep(interval);
 	}
 
 	endwin();
